#!/usr/bin/env perl
use SynBlast;
our $PROGN=[split(/\//, $0)]->[-1];
$VERSION = "SynBlast-" . $VERSION . "\n${PROGN}'s Time-stamp: <2009-08-27 12:35:37 joe>\nsee ${url} for updates.\n";
#######################################################
#doSyntenyAlignment.pl  - part of the SynBlast package for Assisting the Analysis of Conserved Synteny Information
#Copyright (C) 2008  Joerg Lehmann
# 2006-03-04
# joe@bioinf.uni-leipzig.de
#######################################################

=head1 NAME

doSyntenyAlignment.pl - a tool to visualize and align/evaluate blast (tblastn) hits to the order of query proteins.
Query proteins' annotation must be provided as an annotation file in tab-separated format (I<*.protein.info file>, see bottom).
One or more blast hit tabular output files containing only blast hits within a restricted target location and range are required (should represent putative regions of conserved synteny/gene order).

doSyntenyAlignment.pl is part of the L<SynBlast> package for Assisting the Analysis of Conserved Synteny Information.

=head1 SYNOPSIS

B<doSyntenyAlignment.pl> 
B<-i|infoFile=I<Protein.InfoFile>>

[B<-sortByRatio=I<0|1|2|3>>]
[B<-h|htmlFile=I<ResultHtmlFile>>]
[B<-txtFile=I<ResultTxtFile>>]
[B<-n|number=I<NumberOfResultsInHTML>>]
[B<-r|refFields=I<comma-separated-ploci-fields>>]
[B<-p|plociFields=I<comma-separated-ploci-fields>>]
[B<-q|qlociFields=I<comma-separated-qloci-fields>>]
[B<-e|compara[=I<EnsEMBL_DB_init file>]>]
[B<-comparaAll>]
[B<-globalRank>]
[B<-printColors>]
[B<-showAllRegions>]
[B<-s|mean>]

[B<-blastDir=I<tblastnSourceDir>>]
[B<-blastFilePraefix=I<AlternativeBlastFilePraefix>>]
[B<-wublast>]
[B<-dnaQueries>]

[B<-c|sCutOff=I<CutoffBitscore>>]
[B<-o|protOverlap=I<MaxProteinOverlap>>]
[B<-l|lociSizeFactor= I<LociSizeFactor>>]
[B<-mLociSize=I<MaxGlobLociSize>>]
[B<-g|gapCosts=I<gapCost(s)>>]
[B<-m|mismatchCost=I<MismatchCost>>]
[B<-t|misOriPenalty=I<PenaltyFactorMisorientation>>]

[B<-noRelScore>]
[B<-maxQlociDist=I<distance>>]

[B<-ensemblHost=I<EnsEMBL_DB_host>>]
[B<-ensemblUser=I<EnsEMBL_DB_user>>]

[B<-v|verbose>]
[B<--help>] [B<--manual>] [B<--version>]

[[B<-deepChaining>]]
[[B<-snumber=I<NumberOfResultsPerSpeciesInHTML>>]]
[[B<-trackerInputFile=I<TrackerInputFile>>]]
[[B<-trackerN=I<NumberOfExonPerSpecies>>]]
[[B<-trackerFlankS=I<NumberOfNucleotidesToAdd>>]]


SyntenyBlasthitFile(s)

=head1 REQUIRES

L<MySyntenyAln>
L<MySyntenyUtils>,
L<MyUtils>,
L<Getopt::Long>,
L<Pod::Usage>

=head1 DESCRIPTION

The evaluation of synteny regions is done in the following way:

For each protein query found/contained within a synteny region,
the corresponding single blast hits (HSPs) within a limited maximal range (based on the queries genomic range) are stitched together 
covering the protein query more or less (so-called target-loci), and are finally represented by overall sums of Blast bitscores.

A gene order alignment (query loci and target loci positions) is performed for each input synteny region, 
output in HTML or text format contains alignment results and query-target assignments of assembled blast-hit (target) loci 
in rank order according to overall similarity score (alignment score, which is reduced by gap costs), 
or alternatively rank order according to (log)intra-inter-score-ratio sum (see '-sortByRatio' option)

Access to Ensembl databases to display additional information for each assignment can be used (see -compara, -comparaAll options), for comparison.
Determination of the globally-best assembled blast-hit scores (-globalRank option) can be done in order to display the global rank for each assignment.
Note that the original tblastn result files should be available (at least for the reference species; 
see options -blastDir and -blastFilePraefix as well as -globalRank and -noRelScore)

Further options described below.


=head1 ARGUMENTS

=over 5

=item B<-i|infoFile=> I<protein.info-file> [String]

specifies the annotation file containing sequence features (proteins) to be used as reference loci during alignment (reference gene order).
For Ensembl genomes, L<getEnsemblProteins.pl> performs the retrieval automatically.
The format is a tab-separated list of features describing each protein-coding gene of the set, one per line. 
Definition of columns (column names separated by comma) and some more annotation descriptors have to be specified in the head, see following example:

  # protein.info file manually generated
  #!organism=Branchiostoma
  #!coordsystem=scaffold
  #!seqregion=scaffold_89
  #!from=1
  #!to=650000
  #!strand=1
  #!focalgene=89000016
  #!columns=startPos,endPos,geneName,protID,ori,geneID,numExons,protLength
  42268   52505   gene1       fgenesh2_pm.scaffold_89000001   -1      89000001        3       592
  136667  137756  gene2       fgenesh2_pg.scaffold_89000006   1       89000006        2       121
  139176  141632  -       fgenesh2_pg.scaffold_89000007   -1      89000007        5       452
  147110  150418  -       fgenesh2_pg.scaffold_89000009   -1      89000009        1       1103
  #testcomment
  163582  175636  -       fgenesh2_pg.scaffold_89000011   1       89000011        15      649
  ...

=item B<SyntenyBlasthitFile(s)>

The (syntenic) blast hit files to process. Each file is later displayed as an alignment to the reference proteins.
These files should be automatically generated using L<doSyntenyFilter.pl> out of raw blast results in tabular format (either NCBI or WUBLAST format).
The syntenic blast hit files are expected in NCBI-like tabular format.

=head1 OPTIONS

=over 5

=item B<-sortByRatio[=I<0|1|2|3>]>

 To specify the use of ratio sum scores (sum of ratios of intra-/inter-scores), 
 instead of the gene order alignment score, as sorting/ranking criterium for evaluated synteny regions.
 Note that only finally assigned (i.e. assigned in the optimal gene order alignment selected) target loci 
 are taken into account when calculating the alternative scores of a synteny region.
 This can help to better distinguish between orthologs and paralogs.

        0=off,default (sort by gene order alignment score)
	1=ratioSum       (sum of ratios)
	2=logRatioSum    (sum of logarithmized ratios)
	3=focalRatioSum  (sum of ratios weighted with the distance to the focal gene position)

=item B<-h|htmlFile=> I<ResultHtmlFile> [String]

 To output the ranked gene order alignments as tables in html format to the specified file 
 (this also creates a subdirectory for dotplot and alignment graphics as well as assembled blast hits)

=item B<-txtFile=> I<ResultTxtFile> [String]

 To output the ranked gene order alignments as tables in txt format (tab-delimited) to the specified file

=item B<-n|number=> I<NumberOfResultsInHTML> [Integer]

 To limit the number of displayed results in html file.
 This can be used e.g. to only look at the results of  the 10 best evaluated target regions.
 default = 0
 (no limit, i.e. each candidate region is displayed as result, as long as there was any assignment of a target locus possible)


=item B<-r|refFields=> I<comma-separated-ploci-fields> [String]

 To select custom p-loci-data columns displayed/used for the reference overview table in the HTML file
 Note: Column names have to correspond to those used within the annotation file.
	
 default is: "geneName,protID,transID,geneID,protName,chr,startPos,endPos,meanPos,ori,numExons,protLength"
 
 standard/possible field names are (column names created by L<getEnsemblProteins.pl>): 
	chr
	startPos
	endPos
	meanPos (added automatically, if mean flag set)
	protID
	transID
	geneID
	ori
	protLength
	protName
	numExons
	geneName

=item B<-p|plociFields=> I<comma-separated-ploci-fields> [String]

 To select custom ploci-data columns displayed/used for the detailed alignment table in the HTML file
 default is: "protID,geneName,optQloci";
 possible field names are:
	the same names as for the B<-r|refFields=> option +
	optQloci

=item B<-q|qlociFields=> I<comma-separated-qloci-fields> [String]

 To select custom qloci-data columns displayed/used for the detailed alignment table in the HTML file
 default is: "score,chr,startPos,endPos,ori,hits,queryCov,meanIdentity,hitsLength,intra_inter";
 possible field names are:
	score
	ori
	startPos
	endPos
	meanPos (is displayed automatically, if mean flag set)
	chr
	queryCov
	meanIdentity
	hitsCount
	hitsLength
	hits
	compara (is displayed automatically, if compara flag set)
	globalRank (is displayed automatically, if globalRank flag set)
	intra_inter (is displayed automatically, if sortByRatio flag set)

=item B<-e|compara[=I<EnsEMBL_DB_RegistryInitFile> [String]]>

 Flag to enable retrieval and display of Compara results for assigned focal gene loci, 
 i.e. if the target assignment for the focal reference gene (only shown in result overview table) is according to the orthology prediction of the Ensembl Compara database 
 (see 'compara' field in tables).
 Additionally, sequences of Compara-annotated proteins (comparaHits) as well as yet-unannotated (w.r.t. Compara) proteins within the assigned target interval are saved as 
 fasta files into the subdirectory I<<geneID>-homologs>
 Optionally, one can here specify a user-configured Ensembl_DB_RegistryInitFile for use of local Ensembl databases (mysql).
 See http://www.ensembl.org/info/using/api/registry/index.html about the format of such a configuration file.

=item B<-comparaAll>

 Flag to enable display of Compara comparison/result for each assignment in the detailed alignment table. Enables also the B<-compara> flag, if not used explicitely.

=item B<-globalRank[=I<NumberOfBestHitsToReport> [Integer]]>

 Flag to enable the retrieval of globally best hits and display of rank number for each assignment
 (see column 'globalRank' in the detailed alignment table)

=item B<-printColors>

 Flag to generate dotplots graphics with a higher resolution, and additionally as a black-white version

=item B<-showAllRegions>

 Flag to display results for all input regions (even if no alignment/target locus is available); 
 By default, regions with no alignment score are not displayed.



=item B<-blastDir=> I<tblastnSourceDir> [String]

 To specify the directory containing the appropriate tblastn result files,
 needed for the retrieval of the global-best-hit ranks (see option L<-globalRank>).
 The format of file names for the tblastn result files has to be either "QueryID.<SpeciesName>.blastresult" (by default)
 or "<BlastFilePraefix>.<SpeciesName>.blastresult" (if <BlastFilePraefix> is set).
 default = "tblastn" 

=item B<-blastFilePraefix=> I<BlastFilePraefix> [String]

 To specify an alternative blast file praefix and not to look for query specific blast result files,
 but a file named "<BlastFilePraefix>.<SpeciesName>.blastresult"
 Needed for the retrieval of the global-best-hit ranks (see option L<-globalRank>).
 default = undef 
 (There is one query specific blast result file for every single query sequence, named "QueryID.<SpeciesName>.blastresult"

=item B<-wublast>

 Flag to use raw blastresults in WU-BLAST tabular format (instead of NCBI format)
 (only relevant for the raw blast results needed for the globalRank option and the selfscore calculation! Filtered blastresults are always in NCBI-like tabular format!)


=item B<-dnaQueries>

 Flag to use DNA queries instead of proteins, and hence blastn(or similar) instead of tblastn BLAST results


=item B<-c|sCutOff=> I<CutoffBitscore> [Integer]

 To set the minimum bitscore that has to be reached by any assembled blast hit to be considered as target locus.
 default = 50  

=item B<-o|protOverlap=> I<MaxProteinOverlap> [Integer]

 To set the maximal amount of positions that neighbored tblastn hits may overlap on the query site to still be considered as 'consistent'.
 default = 30 
 (amino acid positions) 

=item B<-l|lociSizeFactor=> I<LociSizeFactor> [Float]

 To specify the maximal length allowed for a target locus, relative to the query's range.
 default = 2.0 
 (i.e. twice of the query's range/size is used to restrict the length of a target locus;
  3 or 4 may be a better factor not to miss some more distant exon hits for an assembled hit in some cases)

=item B<-mLociSize=> I<MaxGlobLociSize> [Integer]

 To specify a maximally allowed size (genomic extension) for valid target loci 
 default = 0 (not set)
 if set, the maximal size is restricted, but may be even smaller depending on parameter I<-l> (LociSizeFactor)

=item B<-g|gapCosts=> I<gapCost> [String]

 To specify the gap costs for the gene order alignment, 
 (better to speak of gap scores, as values should be negative here!)
 default = "-2e1:-2e1" (gapP[:gapQ])
 Note: It is possible to specify gap costs for insertions in the target sequence of loci (gapP, i.e. gaps in the reference sequence of loci),
       separately from the gap costs for gaps within the sequence of target loci (gapQ).

=item B<-m|mismatchCost=> I<MismatchCost> [String]

 To specify the score (cost) for a mismatch within gene loci alignment.
 Note: This value should be relatively/highly negative in relation to usually occuring match scores and gap scores, 
 to simply not allow mismatches in optimal alignments!
 default = -1000

=item B<-t|misOriPenalty=> I<PenaltyFactorMisorientation> [Integer]

 To specify the fraction of the matching score that is used in case of misorientated loci matching within the alignment; 
 default = 90 (in percent)

=item B<-maxQlociDist=> I<distance> [String]

 To set the maximal distance between neighbored target loci that are not marked orange/black in the dotplot graphics. 
 (This is just to identify biologically more meaningful blocks of contigous target hits more easily, in case of large-sized synteny regions.)
 default = 500000 nt
 (use 0 for the maximal distance between neighbored reference loci multiplied with the <LociSizeFactor>)

=item B<-rlociSizeFactor=> I<RefLociSizeFactor> [Float]>

 The LociSizeFactor for the reference-score retrieval (needed for selfscore, intra/inter-scores calculations) can be specified, separately.
 default = 2.0
 
 Note: The other parameters used to obtain the best and second best within-reference scores (as needed for the blast-score normalization 
 as well as the intra/inter-score calculations) are fixed: sCutoff=0; protOverlap=40; mLociSize=0.

=item B<-noRelScore>

 Flag to disable the use of relative scores for alignment calculations
 default is: the calculated relative score will be used for alignment calculations 
 Note: This is only possible if within-reference blast hit data is available for all reference protein queries. 
       Otherwise those reference entries will have to be removed from the reference set, manually.


=head2 rarely used options:

=item B<-ensemblHost=> I<EnsEMBL_DB_Hostname> [String]

=item B<-ensemblUser=> I<EnsEMBL_DB_Username> [String]

=item B<-s|mean>

 Flag: using the mean CDS(annotation)/HSP(blasthit) position as sort index
 (default is: using the start and end positions and combine overlapping loci)

=item B<-v|verbose>

 verboseFlag: prints intermediate results verbosely (not recommended)

=item B<-snumber=> I<NumberOfResultsPerSpeciesInHTML> [Integer]

 To limit the number of displayed results for each species in html file. [experimental]
 default = 0
 (no limit)

=item B<-trackerInputFile=> I<TrackerInputFile> [String]

 generates inputfile for use with the Tracker program [experimental]

=item B<-trackerN=> I<NumberOfExonPerSpecies> [Integer]

 number of genes per species to write to tracker input file, 
 default = 0 (no tracker input is generated at all)

=item B<-deepChaining> [experimental]

 Flag to use the recursive version considering all possible combinations of HSPs maximizing the assembled hit 
 w.r.t. query overlap size, range factor, and score cutoff parameters (not recommended)
 default is: using an alignment based approach relying on consistent overlappings of putative exon matches! 
 (faster but might miss some HSPs due to overlap with insignificant HSPs) 



=head1 AUTHOR

Joerg Lehmann, <joe@bioinf.uni-leipzig.de>


=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/SynBlast/


=head1 LICENSE

This  program is free  software; you can redistribute  it and/or
modify it  under the terms of the GNU  General Public License as
published by the  Free Software Foundation; either version 2, or
(at your option) any later version.

This program is  distributed in the hope that it will be useful,
but WITHOUT  ANY WARRANTY; without even the  implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You can view the  GNU General Public License, online, at the GNU
Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.

See also L<perlgpl> for more information.

=head1  SEE ALSO

L<SynBlast::MySyntenyAln>,
L<SynBlast::MySyntenyUtils>, 
L<SynBlast::MyUtils>,
L<getEnsemblProteins.pl>,
L<doBlastJobs.pl>,
L<doSyntenyFilter.pl>,
L<getBestBlastHit.pl>

=cut

use strict;
use warnings;

use SynBlast::MySyntenyAln;
use SynBlast::MyUtils;
use Getopt::Long;
use Pod::Usage;
use SynBlast::EnsEMBLaccess;


$|=1;

my $cmdlineWas = ${main::PROGN} . " " . join(" ", @ARGV);
#$Getopt::Std::STANDARD_HELP_VERSION = 1;

my($infoFile, $htmlFile, $txtFile, $trackFile);
my $htmlLimit=0;
my $htmlSpLimit=0;
my $trackerLimit=0;
my $trackOvl=100;



### some configuration for retrieving globalBestHit (RankData)
my $ablastf=undef;
#$ablastf=$opts{"a"} if(exists($opts{"a"}));
my $bsourceDir="tblastn";
#$sourceDir=$opts{"s"} if(exists($opts{"s"}));
#MyUtils::checkStrEnd(\$sourceDir, "/") && print STDERR "blastresult directory name changed to " . $sourceDir . "\n";



#default values for parameters:
my($scutoff, $protOvl, $globLocSize) = ( 50, 30, 0 );
my($locSizeFac, $rlocSizeFac) = (2.0,  2.0);
my($gapPstr, $mismP, $pMisOri) = ( "-2e1:-2e1", -1e3, 90 );
#output column defaults:
#qfields default is: "score,chr,startPos,endPos,ori,hits,queryCov,meanIdentity,hitsLength,intra_inter";
my($ppfields
   , $pfields
   , $qfields) = ( 
		   join(",", qw(geneName protID transID geneID protName chr startPos endPos meanPos ori numExons protLength)), 
		   join(",", qw(protID geneName optQloci)),
		   join(",", qw(score chr startPos endPos ori hits queryCov meanIdentity hitsLength intra_inter))
		   );

#flags are off per default
my($verboseFl, $bFl, $sFl, $comparaFl, $comparaAllFl, $globalRankFl, $noRelScFl, $ratioFl) = ( 0, 0, 0, undef, undef, undef, 0, undef);
my $deepFl = 0;
my $printcFl=0; ## to use printer-friendly colors in dotplot and alignment graphics
my($ehost, $euser) = ("ensembldb.ensembl.org", "anonymous");
my($help, $version, $man) = (0, 0, 0);
my ($maxDist, $maxDistStr) = (500000, undef);
my $wublastFl=0;
my $showallFl=0;
my $blastnFl=0;

GetOptions( 'help|?' => \$help,	    'version' => \$version,	    'manual' => \$man,
	    'infoFile|i=s' => \$infoFile,
	    'htmlFile|h=s' => \$htmlFile,
	    'txtFile=s' => \$txtFile,
	    'trackerInputFile=s' => \$trackFile,
	    'trackerFlankS=i' => \$trackOvl,

	    'number|n=i' => \$htmlLimit,
	    'snumber=i' => \$htmlSpLimit,
	    'trackerNumber=i' => \$trackerLimit,

	    'blastDir=s' => \$bsourceDir,
	    'blastFilePraefix=s' => \$ablastf,
	
	
	    'sCutOff|c=i' => \$scutoff,
	    'protOverlap|o=i' => \$protOvl,
	    'lociSizeFactor|l=f' => \$locSizeFac,
	    'rlociSizeFactor=f' => \$rlocSizeFac,
	    'mLociSize=i' => \$globLocSize,
	    'gapCosts|g=s' => \$gapPstr,
	    'mismatchCost|m=i' => \$mismP,
	    'misOriPenalty|t=i' => \$pMisOri, ## misorientation penalty factor
	    ##(fraction in percent that is kept of match score in case of wrong orientation wrt reference locus)
	    'maxQlociDist=s' => \$maxDistStr,
	    
	    'refFields|r=s' => \$ppfields,
    	    'plociFields|p=s' => \$pfields,
    	    'qlociFields|q=s' => \$qfields,
	   
	    'verbose|v' => \$verboseFl,
	#    'selfcomp|b' => \$bFl,
	    'mean|s' => \$sFl,
	    'compara|e:s' => \$comparaFl,    # optional: string of initFile to be used for compara-access!!!
	    'comparaAll' => \$comparaAllFl,
			'globalRank:i' => \$globalRankFl,  # optional: integer for nglobal Parameter (levels of global hits to determine)
			'sortByRatio:i' => \$ratioFl,   ## optional: integer 1=sum of ratios (default), 2=log sum of ratios (like product), 3=weighted sum of ratios (distance to focal gene)
			'noRelScore' => \$noRelScFl,
			'deepChaining' => \$deepFl,
				    
	    'ensemblHost=s' => \$ehost,
	    'ensemblUser=s' => \$euser,
	    'printColors' => \$printcFl,
	    'wublast'=> \$wublastFl,
	    'showAllRegions' => \$showallFl,
	    'dnaQueries' => \$blastnFl
	    
#	    'initFile=s' => \$initFile
	    ) 
    or pod2usage(-verbose => 0);
VERSION_MESSAGE() if($version || $help);
HELP_MESSAGE() if $help;
exit(1) if($version);
pod2usage(-verbose => 2, -noperldoc => 1) if $man;
pod2usage(-verbose => 0, -message => "$PROGN: Error - no BlastHitFile(s) specified.\n") unless @ARGV>0;
pod2usage(-verbose => 0, -message => "$PROGN: Error - no ReferenceInfoFile specified (-i option).\n") unless defined($infoFile);

$noRelScFl=1 if($bFl);

#### determine release archive string that the target regions should be tagged with (for valid comparisons with Compara DB of API-release=DB-release version) 
my $relStringShouldBe=lc(SynBlast::EnsEMBLaccess::getRelMonthString());


if(! defined($trackFile)) {
  if(defined($htmlFile)) {
    $trackFile=substr($htmlFile, 0, -4) . "tracker";
  }
}

# for compara comparison
my $initF = undef;
$initF = $comparaFl if(defined($comparaFl) && ($comparaFl ne ""));
$comparaFl=1 if(defined($comparaFl));
$comparaFl=1 if(defined($comparaAllFl));
$comparaAllFl=undef if(! $comparaFl);

my $nglobal = 10;
$nglobal = $globalRankFl if(defined($globalRankFl) && (int($globalRankFl)));
$globalRankFl=1 if(defined($globalRankFl));
#$globalRankFl=1 if(defined($ratioFl));

if(defined($ratioFl))  {
		$ratioFl = 1 if($ratioFl eq "");
} else {
	$ratioFl = 0;
}

$qfields .= ",compara" if( $comparaAllFl && (index($qfields, "compara") == -1));
$qfields .= ",globalRank" if( $globalRankFl && (index($qfields, "globalRank") == -1));

#$qfields .= ",intra_inter" if( $globalRankFl && $ratioFl && (index($qfields, "intra_inter") == -1));
$qfields .= ",intra_inter" if($ratioFl && (index($qfields, "intra_inter") == -1));



my $displRatio;
if(index($qfields, "intra_inter") != -1) {
#	$globalRankFl=1;   ### globalRank flag needed for intra_inter ratio calculation
	 $displRatio=1;
 }


$qfields = "meanPos,${qfields}" if( $sFl && (index($qfields, "meanPos") == -1));


##########################################################################


SynBlast::MySyntenyAln::createSyntenyAlnOutput(
	'-WUBLASTFLAG'=> $wublastFl
	,'-SCUTOFF'=> $scutoff
	,'-PROTOVL'=> $protOvl
	, '-LOCISIZEFAC'=> $locSizeFac
	, '-REF_LOCISIZEFAC'=> $rlocSizeFac
	, '-MLOCISIZE'=> $globLocSize
	, '-MEANFLAG'=> $sFl
	,'-GAPCOSTS'=> $gapPstr
	,'-MISMATCHS'=> $mismP
	, '-MISORIPERC'=> $pMisOri
	, '-NO_RELSCORE'=> $noRelScFl
	, '-SORTFLAG'=> $ratioFl
	, '-REF_FIELDS'=> $ppfields
	, '-PLOCI_FIELDS'=> $pfields
	, '-QLOCI_FIELDS'=> $qfields
	, '-COMPARA_FLAG'=> $comparaFl
	, '-COMPARA_ALL'=> $comparaAllFl
	, '-GLOBAL_RANK_FLAG'=> $globalRankFl
	, '-N_GLOBAL_RANK'=> $nglobal
	, '-PRINT_COLOR_FLAG'=> $printcFl
	, '-MAXQ_LOCI_DIST'=> $maxDistStr
	, '-VERBOSE_FLAG'=> $verboseFl   #0
	, '-DEEP_RECURSION'=> $deepFl
	, '-INFO_FILE'=> $infoFile
	, '-HTML_FILE'=> $htmlFile   
	, '-TXT_FILE'=> $txtFile
	, '-TRACKER_FILE'=> $trackFile
	, '-HTML_LIMIT'=> $htmlLimit
	, '-HTML_SPECIES_LIMIT'=> $htmlSpLimit
	, '-TRACKER_LIMIT'=> $trackerLimit
	, '-TRACKER_OVL'=> $trackOvl
	, '-BLASTFILE_PRAEFIX'=> $ablastf
#	, '-BLASTFILE_REF_PRAEFIX'=> $ablastf  #$aRefBlastf
	, '-BLASTFILE_DIR'=> $bsourceDir
	, '-ENSEMBL_HOST'=> $ehost
	, '-ENSEMBL_USER'=> $euser
	, '-DISPLAY_ALL_REGIONS'=> $showallFl
	, '-ENSEMBL_REL_STR'=> $relStringShouldBe
	, '-REGISTRY_INIT_FILE'=> $initF
	, '-DISPLAY_RATIO_FLAG'=> $displRatio
	, '-SYN_REG_FILES'=> \@ARGV   #\@syntenyRegFiles
	#, '-SILENT_FLAG'=> $silentFl
	, '-CMD_LINE_STR' => $cmdlineWas
	#, '-HTML_FLAG' => 1 ### then the synteny region is expected to be in only one subdirectory level from base dir
	, '-PROGN'=> $PROGN
	, '-VERSION' => $VERSION
	, '-BLASTN_FLAG' => $blastnFl
 );
 
 
 exit(0);
	
#################################################################################################

sub HELP_MESSAGE {
	pod2usage(-verbose => 1);
}
sub VERSION_MESSAGE {
	print $main::VERSION;
} 

#################################################################################################
