#!/usr/bin/env perl
use SynBlast;
our $PROGN=[split(/\//, $0)]->[-1];
$VERSION = "SynBlast-" . $VERSION . "\n${PROGN}'s Time-stamp: <2009-08-27 12:36:46 joe>\nsee ${url} for updates.\n";
#######################################################
#getBestBlastHit.pl  - part of the SynBlast package for Assisting the Analysis of Conserved Synteny Information
#Copyright (C) 2008  Joerg Lehmann
# 2006-06-19
# joe@bioinf.uni-leipzig.de
#######################################################

=head1 NAME

getBestBlastHit.pl - tool to obtain target loci with overall best blastHit scores.

getBestBlastHit.pl is part of the L<SynBlast> package for Assisting the Analysis of Conserved Synteny Information.

=head1 SYNOPSIS

B<getBestBlastHit.pl> 
[B<-iI<Query.InfoFile>>]
[B<-cI<cutoff_bitscore>>]
[B<-oI<Max_QueryOverlap>>]
[B<-D>]
[B<-hI<(EnsEMBL_)OrganismName>>]
[B<-rI<Chr_Contig>>]
[B<-nI<NumberOfLevels>>]
[B<-aI<AlternativeBlastFile>>]
[B<-W I<WU-BLAST_Flag>>]
[B<-v>]
[B<-sI<BLAST-sourceDIR>>]
[B<-lI<SizeFactor>>]
[B<-mI<MaxRegionSize>>]
[B<-e>]
[B<-jI<EnsEMBL_DB_init file>>]
[B<--help>]
[B<--version>]
[B<Blast-QueryIDs (e.g. ENSP000xxxxx)>]

=head1 REQUIRES

require MyUtils;
require MySyntenyUtils;
require MySyntenyAln;
require Getopt::Std;
require Pod::Usage;

=head1 DESCRIPTION

...

=head1 OPTIONS

=over 5

=item B<-i> I<Query.InfoFile> [String]

=item B<-c> I<Cutoff_bitscore> [Float]

default=0

=item B<-o> I<Max_QueryOverlap> [Integer]

in amino acids (or nucleotides, if dnaQuery flag  (option <-D>) is set), default=30

=item B<-D>

to enable alternative use of nucleotide queries and blastn results instead of protein queries/tblastn


=item B<-h> I<(EnsEMBL_)OrganismName> [String]

default is Homo_sapiens

=item B<-r> I<Chromosome/Contig> [String]

restrict search only within certain chromosome/contig

default is: best blast hit of all is found (regardless which chr/contig on)

=item B<-n> I<NumberOfLevels> [Integer]

prints more than only the first best blast hit(s)

default is: 1 (only the best assembled blast hit/loci is listed)

=item B<-a> I<AlternativeBlastFileName> [String]

specifies in which file to look for blasthits.

default is: ${blastQID}*${organismN}.blastresult

=item B<-W>

for use of tabular blastresults in WU-BLAST format (different columns and orders). 

=item B<-v>


VerboseFlag: prints results verbosely

=item B<-s> I<BLAST-sourceDIR> [String]

specify the source dir for the BLAST-result-files to be used
(default is: ./tblastn)

=item B<-l> I<SizeFactor> [Integer]

specify a multiple of the reference cds range/size allowed for resulting assembled blast hits
(default is: 2)

=item B<-m> I<MaxRegionSize> [Integer]

specify a maximal allowed size (genomic extension) for allowed assembled blast hit loci (default is 0 (not set)),
if set, the maximal size is restricted, but may be even smaller depending on parameter -l (SizeFactor)

=item B<-e>


Flag: check if result for reference gene is according to Orthology-Annotation of EnsEMBL-Compara database

=item B<-j> I<EnsEMBL_DB_init file> [String]

only if -e option set; set optional ensembl database init file     

default = automatic retrieving via 'ensembldb.ensembl.org'

see http://www.ensembl.org/info/software/registry/index.html

=item B<-w>


Flag: using all queryIDs out of info-file, regardless which queryIDs are submitted via command line

=item B<Blast-QueryID(s)>

The queryIDs to use to obtain the best assembled blastHit for.
(if missing, the reference protein ID(s) are used automatically)

=head1 AUTHOR

Joerg Lehmann, <joe@bioinf.uni-leipzig.de>


=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/SynBlast/


=head1 LICENSE

This  program is free  software; you can redistribute  it and/or
modify it  under the terms of the GNU  General Public License as
published by the  Free Software Foundation; either version 2, or
(at your option) any later version.

This program is  distributed in the hope that it will be useful,
but WITHOUT  ANY WARRANTY; without even the  implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You can view the  GNU General Public License, online, at the GNU
Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.

See also L<perlgpl> for more information.

=head1  SEE ALSO

L<SynBlast::MySyntenyAln>,
L<SynBlast::MySyntenyUtils>, 
L<SynBlast::MyUtils>,
L<getEnsemblProteins.pl>
L<doBlastJobs.pl>
L<doSyntenyFilter.pl>,
L<doSyntenyAlignment.pl>,

=cut

use strict;
use warnings;

use SynBlast::MySyntenyAln;
use Getopt::Std;
use Pod::Usage;
use SynBlast::EnsEMBLaccess;

my $cmdlineWas = ${main::PROGN} . " " . join(" ", @ARGV);
$Getopt::Std::STANDARD_HELP_VERSION = 1;
my %opts=();
getopts('i:c:o:a:vs:h:r:n:ej:wl:s:dWD', \%opts) || pod2usage(-verbose => 0);
#pod2usage(-verbose => 0, -message => "$PROGN: Error - no Blast-QueryID(s) specified.\n") unless @ARGV>0;
pod2usage(-verbose => 0, -message => "$PROGN: Error - no ReferenceInfoFile specified (-i option).\n") unless exists($opts{"i"});


my $wublastFl=0;
$wublastFl = 1 if(exists($opts{"W"}));


my $infoFile=$opts{"i"};

my $scutoff=0;
if(exists($opts{"c"})) {
    $scutoff = int($opts{"c"});
}
my $protOvl=30;
if(exists($opts{"o"})) {
    $protOvl = int($opts{"o"});
}

my $verboseFl=0;
$verboseFl=1 if(exists($opts{"v"}));

my $deepFl=0;
$deepFl=1 if(exists($opts{"d"})); ## use old recursion of calcSplAln

my $dnaQueryFl=0;
$dnaQueryFl=1 if(exists($opts{"D"})); ## for use of DNA-Queries and blastn BLAST results


my $rangeLimit=2; # default value is 2: twice the size of reference gene is allowed for size of assembled blast hit
$rangeLimit=$opts{"l"} if(exists($opts{"l"}));
#print "SizeLimitFactor=${rangeLimit}\n";

my $maxGlobSize=0; # default value is 0: no additional restriction (only the reference dependent rangeLimit)
$maxGlobSize=int($opts{"m"}) if(exists($opts{"m"}));

my $comparaFl=0;
$comparaFl=1 if(exists($opts{"e"}));
my $cinitFile=undef;
$cinitFile=$opts{"j"} if(exists($opts{"j"}));

my $organismN="Homo_sapiens";
$organismN=$opts{"h"} if(exists($opts{"h"}));

my $restrChr=undef;
$restrChr=$opts{"r"} if(exists($opts{"r"}));

my $nbest=1;
$nbest=$opts{"n"} if(exists($opts{"n"}));

my $ablastf=undef;
$ablastf=$opts{"a"} if(exists($opts{"a"}));

my $allQueriesFl=0;
$allQueriesFl=1 if(exists($opts{"w"}));

my $sourceDir="tblastn";
$sourceDir=$opts{"s"} if(exists($opts{"s"}));
SynBlast::MyUtils::checkStrEnd(\$sourceDir, "/") && print STDERR "blastresult directory name changed to " . $sourceDir . "\n";

# creating list of exons out of infoFile...
my @exons=();
my %queryIdx=(); # stores the position of query-key in exon-list
my @refExons=();
my @exonList=();

my($icolIdx
	, $infoH
	, $realFlankS
	, $realSlength) = SynBlast::MySyntenyUtils::infofile2Exonlist("${infoFile}"
																											, \@exons
																											, \%queryIdx
																											, \@refExons
																											, \@exonList
																											, 0
																											, "protID"
																											);
my @infoData = ( \@exons, \%queryIdx, \@refExons, \@exonList, $infoH, $icolIdx, $realFlankS, $realSlength);



if($allQueriesFl) {
    @ARGV = @exonList;
    print STDERR "Note: Using all available queries out of infofile...\n";
} elsif(! @ARGV) {
    @ARGV = @refExons;
    print STDERR "Warning: No Query-IDs specified. Using reference query id(s) (";
    print STDERR join(", ", @refExons) . ")\n";
#    exit 0;
}

my $sliceAdapt;
my $member_adaptor;
my $homologyAdapt;
my $comparaData=undef;
if($comparaFl) {
	SynBlast::EnsEMBLaccess::loadRegistry($cinitFile, undef, undef);
	$member_adaptor = SynBlast::EnsEMBLaccess::getDBadaptor("Multi", "compara", "Member");
  $homologyAdapt = SynBlast::EnsEMBLaccess::getDBadaptor('Multi', 'compara', 'Homology');
	$sliceAdapt = SynBlast::EnsEMBLaccess::getDBadaptor($organismN, "core", "Slice");	
	%{$comparaData}=();
}
for(my $ai=0; $ai<@ARGV; $ai++) { # process all Blast-QueryIDs
    my $blastQID=$ARGV[$ai];
    if(! defined($queryIdx{$blastQID})) {
	warn "# ${blastQID} could not be found in infofile! Skipped!\n";
	next;
    }
    my $QgeneID = $exons[$queryIdx{$blastQID}]->[$icolIdx->{'geneID'}]; # get the corresponding geneID for current blastQID  #$prot2geneID->{$blastQID};
    my $QgeneName = $exons[$queryIdx{$blastQID}]->[$icolIdx->{'geneName'}];
    print "# ${blastQID} (" . $QgeneName  . ", " . $QgeneID . ") against ${organismN}:\n";

    if($comparaFl) { #### pre-save homologues from compara...
   		my $cmember=$member_adaptor->fetch_by_source_stable_id('ENSEMBLGENE', $QgeneID);
		  die "ERROR: could not get member for ${QgeneID}!\n" unless (defined $cmember);
		  ### query compara db
			SynBlast::EnsEMBLaccess::storeHomologsForGID($cmember, $cmember->stable_id(), $organismN, $comparaData, $homologyAdapt);
   	}
   	
    my $blastFile;
    if(defined($ablastf)){
	$blastFile= "${sourceDir}/" . $ablastf;
	$blastFile.= ".${organismN}.blastresult" if(substr($ablastf, -12) ne ".blastresult");
    } else {
	$blastFile = "${sourceDir}/${blastQID}*${organismN}.blastresult";
    }
    
    my @blastFiles= glob($blastFile);
    die "Error: ${blastFile} is not unique or could not be found!\n" unless (@blastFiles == 1);
    #~ SynBlast::MySyntenyAln::getBestBlastLoci(\@infoData
									#~ , $blastFiles[0]
									#~ , $blastQID
									#~ , $scutoff
									#~ , $protOvl
									#~ , $verboseFl +1
									#~ , $restrChr
									#~ , $nbest
									#~ , $comparaFl
									#~ , $organismN
									#~ , $QgeneID
									#~ , $QgeneName
									#~ , $rangeLimit
									#~ , $deepFl
									#~ , $sliceAdapt
									#~ , $comparaData->{$organismN}
									#~ , $maxGlobSize
									#~ , $wublastFl
									#~ );
									
									
	SynBlast::MySyntenyAln::getBestBlastLoci( '-INFO_DATA' => \@infoData
										, '-BLAST_FILE' => $blastFiles[0]
										, '-QUERY_ID' => $blastQID
										,  '-SCUTOFF' => $scutoff
										,  '-MAX_OVL' => $protOvl
										,  '-VERBOSE_FLAG' => $verboseFl +1
										, '-RESTR_CHR' => $restrChr
										,  '-NBEST' => $nbest
										, '-COMPARA_FLAG' => $comparaFl
										, '-TARGET_NAME' => $organismN
										, '-QUERY_GENE_ID' => $QgeneID
										,  '-QUERY_GENE_NAME' => $QgeneName
										, '-LOCUS_RANGE_FACT' => $rangeLimit
										, '-DEEP_FLAG' => $deepFl 
										, '-SLICE_ADAPTOR' => $sliceAdapt
										, '-COMPARA_DATA' => $comparaData->{$organismN}
										, '-MAX_GLOBAL_LOCUS_SIZE' => $maxGlobSize
										, '-WUBLAST_FLAG' => $wublastFl
										, '-BLASTN_FLAG' => $dnaQueryFl
								);
									
}	


exit 0;

#################################################################################################

sub HELP_MESSAGE {
	pod2usage(-verbose => 1);
}
sub VERSION_MESSAGE {
	print $main::VERSION;
} 

################################################################################
