#!/usr/bin/env perl
use SynBlast;
our $PROGN=[split(/\//, $0)]->[-1];
$VERSION = "SynBlast-" . $VERSION . "\n${PROGN}'s Time-stamp: <2009-08-28 15:16:22 joe>\nsee ${url} for updates.\n";
########################################################
#getEnsemblProteins.pl  - part of the SynBlast package for Assisting the Analysis of Conserved Synteny Information
#Copyright (C) 2008  Joerg Lehmann
# 2006-02-08
# joe@bioinf.uni-leipzig.de
#######################################################

=head1 NAME

getEnsemblProteins.pl - a tool to retrieve syntenic/contiguous sets of protein-coding genes from an Ensembl genome database 
in form of an annotation file (.protein.info suffix) and protein sequence files (fasta format). The gene/protein set is determined 
via a focal gene of interest, together with its flanking protein-coding genes (flanking region as specified by the user).

getEnsemblProteins.pl is part of the L<SynBlast> package for Assisting the Analysis of Conserved Synteny Information.


=head1 SYNOPSIS

B<getEnsemblProteins.pl> 
[B<-fI<FlankingSize>>]
[B<-nI<MaxNumberOfFlankingProteins>>]
[B<-iI<IntervalStart[:IntervalEnd]_position>>]
[B<-lI<contig/chromosome>>]
[B<-dI<ProteinDir>>]
[B<-hI<EnsEMBL_OrganismName>>]
[B<-cI<EnsEMBL_DB_init file>>]
[B<-eI<EnsEMBL_DB_host>>]
[B<-uI<EnsEMBL_DB_user>>]
[B<-s>]
[B<-o>]
[B<--help>]
[B<--version>]
EnsEMBL_ID(s)

=head1 REQUIRES

require Getopt::Std;
require Pod::Usage;
require Bio::EnsEMBL::DBSQL::DBAdaptor;

=head1 DESCRIPTION

getEnsemblProteins.pl retrieves annotation data and protein sequences for a focal gene's flanking region 
from Ensembl Core databases.

=head1 OPTIONS

=over 5

=item B<-f> I<FlankingSize> [String]

flanking size around selected gene, default=1e6

=item B<-i> I<IntervalStart[:IntervalEnd]_position> [String]

additionally, sequence interval can be specified (start and end positions) containing hopefully selected gene, default=not used;
the flanking size param is then used to limit the max size of the interval to search on (started from start position of interval)

=item B<-l> I<chromosome/contig> [String]

given a chromosome/contig name in combination with sequence interval (option -i), it displays only the proteins within the interval and does not save/download anything

=item B<-n> I<MaxNumberOfFlankingProteins> [Integer]

Maximum number of flanking genes for target gene to be used (default=0 => all within flanking size or sequence interval are used).

=item B<-d> I<proteinDir> [String]

save fasta files in proteinDir
default proteinDir is {geneID}_{flankS}.protein.dir

=item B<-h> I<EnsEMBL_OrganismName> [String]

default: Species name is automatically selected based on the first geneID argument (and Homo_sapiens if the chromosome-interval-option is set)

the correspondent database for the API release (e.g. 36) is then chosen automatically or as specified in I<EnsEMBL_DB_init file>.

=item B<-c> I<EnsEMBL_DB_init file> [String]

default behaviour => automatic retrieving via 'ensembldb.ensembl.org'

see http://www.ensembl.org/info/software/registry/index.html


=item B<-s>


downloads/displays all coding transcripts within region, even if overlapping (for manually editing)
(default is: only the longest protein-transcript for each overlapping transcript groups and strand is downloaded, and further transcripts are only displayed as comment (#))

=item B<-o>


additionally download protein sequences as fasta files (only entries that are not comment out are used)

=item B<EnsEMBL_ID(s)>

The gene ID(s) to use.

=head1 AUTHOR

Joerg Lehmann, <joe@bioinf.uni-leipzig.de>


=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/SynBlast/


=head1 LICENSE

This  program is free  software; you can redistribute  it and/or
modify it  under the terms of the GNU  General Public License as
published by the  Free Software Foundation; either version 2, or
(at your option) any later version.

This program is  distributed in the hope that it will be useful,
but WITHOUT  ANY WARRANTY; without even the  implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You can view the  GNU General Public License, online, at the GNU
Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.

See also L<perlgpl> for more information.

=head1  SEE ALSO

L<SynBlast::MySyntenyAln>,
L<SynBlast::MySyntenyUtils>, 
L<SynBlast::MyUtils>,
L<doBlastJobs.pl>
L<doSyntenyFilter.pl>,
L<doSyntenyAlignment.pl>,
L<getBestBlastHit.pl>

=cut

use strict;
use warnings;

use Getopt::Std;
use Pod::Usage;
#use Bio::EnsEMBL::DBSQL::DBAdaptor;
use SynBlast::MyUtils;
use SynBlast::EnsEMBLaccess;

$Getopt::Std::STANDARD_HELP_VERSION = 1;
my %opts=();
getopts('f:n:d:osc:h:e:u:i:l:', \%opts) || pod2usage(-verbose => 0);

my $chromosome = undef;
$chromosome=$opts{"l"} if(exists($opts{"l"}));

# constraints on gene length
#my $maxGL=20000;
#my $minGL=100;
# constraints on length of transcript (maximum number of exons)
#my $maxExons=6;

my $flankS="1e6";
my $flankMP=0;
$flankMP=$opts{"n"} if(exists($opts{"n"}));

my $ensemblVstr=undef; #"dec2006";
#$ensemblVstr=lc($opts{"x"}) if(exists($opts{"x"}));
$ensemblVstr = lc(SynBlast::EnsEMBLaccess::getRelMonthString()) unless($ensemblVstr);

my $intvStr=undef;
$intvStr=$opts{"i"} if(exists($opts{"i"}));
my($intvS, $intvE) = (undef, undef);
($intvS, $intvE) = split(/:/, $intvStr) if(defined($intvStr));
if(defined($intvStr)) {
    if(defined($intvE)) { 
	pod2usage(-verbose => 0, -message => "${PROGN}: Error - sequence interval is not correct!\n") unless($intvE>=$intvS);
    }
}

if(! defined($chromosome)) {
    pod2usage(-verbose => 0, -message => "${PROGN}: Error - no EnsEMBL_ID(s) specified.\n") unless @ARGV>0;
} else {
    pod2usage(-verbose => 0, -message => "${PROGN}: Error - no (correct) sequence interval specified for chromosome/contig ${chromosome}!\n") unless(defined($intvE) && ($intvE>=$intvS));
    print "Ignoring geneIDs...\n" if(@ARGV);
}


my $skipFlag=1;
$skipFlag=0 if(exists($opts{"s"}));

my ($host, $user) = (undef, undef); # standard web location (ensembldb.ensembl.org is used automatically, if no initfile set and no other host specified)
$host = $opts{"e"} if(exists($opts{"e"}));
$user = $opts{"u"} if(exists($opts{"u"}));



$flankS = ("" . $opts{"f"}) if(exists($opts{"f"}));
my $saveFlag=0;
$saveFlag = 1 if(exists($opts{"o"}));
my $subDIR;
if(exists($opts{"d"})) {
    $subDIR = $opts{"d"}; 
    SynBlast::MyUtils::checkStrEnd(\$subDIR, "/"); # && print "protein fasta directory name changed to " . $subDIR . "\n";
}

my $initF = undef;
$initF = $opts{"c"} if(exists($opts{"c"}));

#my $orgName = "Homo_sapiens";
my $orgName;
$orgName = $opts{"h"} if(exists($opts{"h"}));


SynBlast::EnsEMBLaccess::loadRegistry($initF, $host, $user);

if(@ARGV && (! $orgName)) {
  my($rspecies, $rtype) = Bio::EnsEMBL::Registry->get_species_and_object_type($ARGV[0]);
   $orgName = $rspecies;
}
$orgName = "Homo_sapiens" unless($orgName);

my $slice_adaptor = SynBlast::EnsEMBLaccess::getDBadaptor($orgName, "core", "Slice");

if(defined($chromosome)) {
    ### only display interval...
    my $coordsys="";
    $coordsys = "chromosome" if(int($chromosome));
    my $slice = $slice_adaptor->fetch_by_region($coordsys, $chromosome, $intvS, $intvE);
    SynBlast::EnsEMBLaccess::scanSlice('-SLICE' => $slice
									, '-SAVE_FLAG' => 0
									#, undef, undef, undef, undef, 
									,  '-INTERVAL_START' => $intvS
									, '-INTERVAL_END' => $intvE
									, '-SKIP_FLAG' => $skipFlag
									, '-ENSEMBL_VERSION_STR' => $ensemblVstr
									, '-ORG_NAME' => $orgName
									);
} else {
    ### display and download interval around specified geneIDs...
    my $gene_adaptor= SynBlast::EnsEMBLaccess::getDBadaptor($orgName, "core", "Gene");
    while(@ARGV) {
	   my $geneID=shift(@ARGV);
	   my $focalgene = $gene_adaptor->fetch_by_stable_id($geneID);
	   if(! defined($focalgene)) {
	    print STDERR "ERROR: Could not fetch ${geneID}, will be skipped!\n";
	    next;
	   }
	   my $fgeneName="";
	   $fgeneName= ($focalgene->external_name() . "_") if(defined($focalgene->external_name()));
	   my $infoF="${fgeneName}${geneID}_${flankS}";
	   $infoF .= "_${intvS}_" . (defined($intvE) ? "${intvE}" : "undef") #. "_${flankMP}" 
	                                     if(defined($intvStr));
     $infoF .= ".protein";
	   $subDIR = "${infoF}.dir" if(! exists($opts{"d"}));
	   $infoF .= ".info";
	   my $slice = $slice_adaptor->fetch_by_gene_stable_id($geneID, $flankS);
	   SynBlast::EnsEMBLaccess::scanSlice( '-SLICE' => $slice
									, '-SAVE_FLAG' => $saveFlag
									,  '-GID' => $geneID
									, '-FLS' => $flankS
									, '-INFO_FILE' => $infoF
									, '-SUB_DIR' => $subDIR
									,  '-INTERVAL_START' => $intvS
									, '-INTERVAL_END' => $intvE
									, '-SKIP_FLAG' => $skipFlag
									, '-ENSEMBL_VERSION_STR' => $ensemblVstr
									, '-ORG_NAME' => $orgName
								);  ##$slice, $saveFlag, $geneID, $flankS, $infoF, $subDIR, $intvS, $intvE, $skipFlag);
	
    }
}
exit 0;
############################
#################################################################################################

sub HELP_MESSAGE {
	pod2usage(-verbose => 1);
}
sub VERSION_MESSAGE {
	print $main::VERSION;
} 

################################################################################
